//
//  GalleryView.m
//  EmojiKeyboardSample
//
//  Created by Meenakshi on 11/11/13.
//  Copyright (c) 2013 Mindfire Solutions. All rights reserved.
//

#define kPageTag 5000

#define	_ReleaseObject(obj)					if (obj) { [obj release]; obj = nil; }

#import "GalleryView.h"

@interface GalleryView (private)

- (void) manageScrollViewFromPage:(NSInteger)firstNeededPageIndex toPage:(NSInteger)lastNeededPageIndex;

@end

@implementation GalleryView

@synthesize imageArray = _imageArray;

@synthesize galleryViewDelegate;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
    }
    return self;
}

#pragma mark - 
#pragma mark SmoothingScrollView

- (EmoticansView *)dequeueRecycledPage
{
    EmoticansView *page = [_recycledPages anyObject];
    page.delegate = nil;
    return page;
}

- (BOOL)isDisplayingPageForIndex:(NSUInteger)index
{
    BOOL pageFound = NO;
    
	for (EmoticansView* page in _visiblePages) 
	{
        if (page.tag == index)
		{
            pageFound = YES;
            break;
        }
    }
    return pageFound;
}

- (void)configurePage:(EmoticansView *)page forIndex:(NSUInteger)index
{
	NSLog(@"Currentpage %d", index);
	self.pageControl.currentPage = index;
	
    page.tag = index;
    page.frame = CGRectMake(index*_viewSize.width, 0, _viewSize.width, _viewSize.height);
	
	NSUInteger startIndex = index*(kEmojiKeyboardColumn*kEmojiKeyboardRow - 1);
	NSUInteger endIndex = startIndex + kEmojiKeyboardColumn*kEmojiKeyboardRow - 1;
	[page cleanEmoticanView];
	[page loadEmoticansFromIndex:startIndex toIndex:endIndex];
}

- (void) loadPages
{
	CGRect visibleBounds = self.bounds;
	
	int firstNeededPageIndex = floorf(CGRectGetMinX(visibleBounds) / CGRectGetWidth(visibleBounds));
	int lastNeededPageIndex  = floorf((CGRectGetMaxX(visibleBounds)-1) / CGRectGetWidth(visibleBounds));
	
	firstNeededPageIndex = MAX(firstNeededPageIndex, 0);
	lastNeededPageIndex  = MIN(lastNeededPageIndex, _imageArray.count - 1);
    
	for (EmoticansView *page in _visiblePages)
	{
        if (page.tag < firstNeededPageIndex || page.tag > lastNeededPageIndex)
		{
            [_recycledPages addObject:page];
            [page removeFromSuperview];
        }
    }
    [_visiblePages minusSet:_recycledPages];
    
    [self manageScrollViewFromPage:firstNeededPageIndex toPage:lastNeededPageIndex];
}

- (void) manageScrollViewFromPage:(NSInteger)firstNeededPageIndex toPage:(NSInteger)lastNeededPageIndex
{
    for (int displayIndex = firstNeededPageIndex; displayIndex <= lastNeededPageIndex; displayIndex++) 
	{
        if (![self isDisplayingPageForIndex:displayIndex])
		{
            EmoticansView* page = [[EmoticansView alloc] initWithFrame:CGRectMake(displayIndex*_viewSize.width, 0, _viewSize.width, _viewSize.height)];
            
            page.delegate = self.galleryViewDelegate;
            page.tag = displayIndex + kPageTag;
			page.backgroundColor = [UIColor clearColor];

            [self configurePage:page forIndex:displayIndex];
            [self addSubview:page];
            [_visiblePages addObject:page];
        }
    }  
}

#pragma mark -
#pragma scrollView delegate methods

- (void)scrollViewDidScroll:(UIScrollView *)sender
{
    [self loadPages];
}

#pragma mark -

- (void) initialSetUpForScrollViewWithNeededFrames:(NSInteger) noOfFrames
{
    self.pagingEnabled = YES;
	self.backgroundColor = [UIColor clearColor];
	
	self.pageControl.numberOfPages = noOfFrames;
	self.pageControl.currentPage = 0;
	[self.pageControl addTarget:self action:@selector(changePage:) forControlEvents:UIControlEventValueChanged];
	   
    _ReleaseObject(_recycledPages);
    _recycledPages = [[NSMutableSet alloc] init];
    _ReleaseObject(_visiblePages);
    _visiblePages  = [[NSMutableSet alloc] init];
    
    _ReleaseObject(_imageArray);
    _imageArray = [[NSMutableArray alloc] initWithCapacity:noOfFrames];
    
    for (int i =0; i < noOfFrames; ++i)
        [_imageArray addObject:@""];
    
    _viewSize = self.frame.size;
    
    [self setContentSize:CGSizeMake(_viewSize.width*noOfFrames, _viewSize.height)];
    
    self.delegate = self;
    
    [self loadPages];
}

- (IBAction) changePage:(id)sender
{
	CGRect frame = self.frame;
    frame.origin.x = frame.size.width * self.pageControl.currentPage;
    frame.origin.y = 0;
    [self scrollRectToVisible:frame animated:TRUE];
    NSLog(@"%i", self.pageControl.currentPage);
}



@end
